"""Takes a bunch of Rougail XML dispatched in differents folders
as an input and outputs a Tiramisu's file.

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Sample usage::

    >>> from rougail import Rougail
    >>> rougail = Rougail('/usr/share/rougail/rougail.dtd')
    >>> rougail.create_or_populate_from_xml(['/usr/share/rougail/dicos'])
    >>> rougail.create_or_populate_from_xml(['/usr/share/rougail/extra1'], 'extra1')
    >>> rougail.space_visitor('/usr/share/rougail/funcs.py')
    >>> tiramisu = rougail.save()

The Rougail

- loads the XML into an internal RougailObjSpace representation
- visits/annotates the objects
- dumps the object space as Tiramisu string

The visit/annotation stage is a complex step that corresponds to the Rougail
procedures.
"""

from typing import List
from .objspace import RougailObjSpace
from .xmlreflector import XMLReflector
from .tiramisureflector import TiramisuReflector
from .annotator import SpaceAnnotator


class Rougail:
    """Rougail object
    """
    def __init__(self,
                 dtdfilename: str,
                 ) -> None:
        self.xmlreflector = XMLReflector()
        self.xmlreflector.parse_dtd(dtdfilename)
        self.rougailobjspace = RougailObjSpace(self.xmlreflector)
        self.funcs_path = None

    def create_or_populate_from_xml(self,
                                    xmlfolders: List[str],
                                    namespace: str=None,
                                    ) -> List[str]:
        """Parses a bunch of XML files and populates the RougailObjSpace
        """
        for xmlfile in self.xmlreflector.load_xml_from_folders(xmlfolders):
            document = self.xmlreflector.parse_xmlfile(xmlfile)
            self.rougailobjspace.redefine_variables = []
            self.rougailobjspace.xml_parse_document(xmlfile,
                                                    document,
                                                    self.rougailobjspace.space,
                                                    namespace,
                                                    )

    def space_visitor(self,
                      eosfunc_file: str,
                      ) -> None:
        """All XML are loader, now annotate content
        """
        self.funcs_path = eosfunc_file
        SpaceAnnotator(self.rougailobjspace, eosfunc_file)

    def save(self) -> str:
        """Return tiramisu object declaration as a string
        """
        tiramisu_objects = TiramisuReflector(self.rougailobjspace,
                                             self.funcs_path,
                                             )
        return tiramisu_objects.get_text() + '\n'
